#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""TODO"""


# This file is part of Linshare cli.
#
# LinShare cli is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# LinShare cli is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LinShare cli.  If not, see <http://www.gnu.org/licenses/>.
#
# Copyright 2019 Frédéric MARTIN
#
# Contributors list :
#
#  Frédéric MARTIN frederic.martin.fma@gmail.com
#


import traceback

from argparse import RawTextHelpFormatter
from argcomplete import debug
from linshareapi.cache import Time
from argtoolbox import DefaultCompleter as Completer
from vhatable.filters import PartialOr
from vhatable.cell import ComplexCellBuilder
from linsharecli.user.core import DefaultCommand as Command
from linsharecli.common.core import add_list_parser_options
from linsharecli.common.core import add_delete_parser_options
from linsharecli.common.tables import TableBuilder
from linsharecli.common.tables import DeleteAction
from linsharecli.common.actions import CreateOneAction
from linsharecli.common.actions import UpdateOneAction


class DefaultCommand(Command):
    """TODO"""

    IDENTIFIER = "creationDate"

    MSG_RS_UPDATED = (
        "The shared space member '%(account)s' "
        "(%(uuid)s) was successfully updated."
    )
    MSG_RS_CREATED = (
        "The shared space member '%(account)s' "
        "(%(uuid)s) was successfully created."
    )

    CFG_DELETE_MODE = 1
    CFG_DELETE_ARG_ATTR = "ss_uuid"

    def complete(self, args, prefix):
        super().__call__(args)
        json_obj = self.ls.shared_spaces.list()
        return (v.get(self.RESOURCE_IDENTIFIER)
                for v in json_obj if v.get(
                    self.RESOURCE_IDENTIFIER).startswith(prefix))

    def complete_shared_spaces(self, args, prefix):
        """TODO"""
        super().__call__(args)
        json_obj = self.ls.shared_spaces.list()
        return (v.get(self.RESOURCE_IDENTIFIER)
                for v in json_obj if v.get(
                    self.RESOURCE_IDENTIFIER).startswith(prefix))


class SharedSpaceCompleter:
    """TODO"""
    # pylint: disable=too-few-public-methods

    def __init__(self, config):
        self.config = config

    def __call__(self, prefix, **kwargs):
        try:
            debug("\n------------ SharedSpaceCompleter -----------------")
            debug("Kwargs content :")
            for i, j in list(kwargs.items()):
                debug("key : " + str(i))
                debug("\t - " + str(j))
            debug("\n------------ SharedSpaceCompleter -----------------\n")
            args = kwargs.get('parsed_args')
            cmd = DefaultCommand(self.config)
            return cmd.complete_shared_spaces(args, prefix)
        # pylint: disable=broad-except
        except Exception as ex:
            debug("\nERROR:An exception was caught :" + str(ex) + "\n")
            traceback.print_exc()
            debug("\n------\n")
            return ["comlete-error"]


class ListCommand(DefaultCommand):
    """ List all shared_spaces store into LinShare."""

    @Time('linsharecli.shared_spaces', label='Global time : %(time)s')
    def __call__(self, args):
        super().__call__(args)
        endpoint = self.ls.shared_spaces.members
        tbu = TableBuilder(self.ls, endpoint, self.DEFAULT_SORT)
        tbu.load_args(args)
        tbu.add_custom_cell(
            "role",
            ComplexCellBuilder(
                '{name}\n({uuid:.8})',
                '{name} ({uuid:})',
                '{name}',
            )
        )
        tbu.add_custom_cell(
            "nestedRole",
            ComplexCellBuilder(
                '{name}\n({uuid:.8})',
                '{name} ({uuid:})',
                '{name}',
            )
        )
        tbu.add_custom_cell(
            "account",
            ComplexCellBuilder(
                '{name}\n({uuid:.8})',
                '{name} <{mail}> ({uuid})',
                '{name}',
            )
        )
        tbu.add_custom_cell(
                "node",
                ComplexCellBuilder('{nodeType}: {name} ({uuid:.8})'))
        tbu.add_action('delete', DeleteAction(
            mode=self.CFG_DELETE_MODE,
            parent_identifier=self.CFG_DELETE_ARG_ATTR
        ))
        tbu.add_filters(
            PartialOr("account", args.accounts, True),
            PartialOr(self.RESOURCE_IDENTIFIER, args.uuids, True),
            PartialOr("role", args.roles, True)
        )
        return tbu.build().load_v2(endpoint.list(args.ss_uuid)).render()

    def complete_fields(self, args, prefix):
        """TODO"""
        # pylint: disable=unused-argument
        super().__call__(args)
        cli = self.ls.shared_spaces
        return cli.get_rbu().get_keys(True)


class CreateCommand(DefaultCommand):
    """TODO"""

    @Time('linsharecli.shared_spaces', label='Global time : %(time)s')
    def __call__(self, args):
        super().__call__(args)
        act = CreateOneAction(self, self.ls.shared_spaces.members)
        act.load(args)
        act.rbu.set_value('node', {'uuid': args.ss_uuid})
        return act.execute()


class CreateDriveCommand(DefaultCommand):
    """TODO"""

    @Time('linsharecli.shared_spaces', label='Global time : %(time)s')
    def __call__(self, args):
        super().__call__(args)
        act = CreateOneAction(self, self.ls.shared_spaces.members.drives)
        act.load(args)
        act.rbu.set_value('node', {'uuid': args.ss_uuid})
        return act.execute()


class DeleteCommand(DefaultCommand):
    """Delete shared space."""

    @Time('linsharecli.shared_spaces', label='Global time : %(time)s')
    def __call__(self, args):
        super().__call__(args)
        act = DeleteAction(
            mode=self.CFG_DELETE_MODE,
            parent_identifier=self.CFG_DELETE_ARG_ATTR
        )
        act.init(args, self.ls, self.ls.shared_spaces.members)
        return act.delete(args.uuids)


class UpdateCommand(DefaultCommand):
    """TODO"""

    @Time('linsharecli.shared_spaces', label='Global time : %(time)s')
    def __call__(self, args):
        super().__call__(args)
        self.check_required_options(
            args,
            ['role'],
            ["--role"])
        endpoint = self.ls.shared_spaces.members
        node = endpoint.get(args.ss_uuid, args.uuid)
        act = UpdateOneAction(self, endpoint)
        rbu = endpoint.get_rbu()
        rbu.copy(node)
        rbu.load_from_args(args)
        return act.load(args).execute(rbu.to_resource())


def add_parser(subparsers, name, desc, config):
    """TODO"""
    parser_tmp = subparsers.add_parser(name, help=desc)
    parser_tmp.add_argument(
        'ss_uuid',
        help="shared_space uuid"
        ).completer = SharedSpaceCompleter(config)
    subparsers2 = parser_tmp.add_subparsers()

    # command : list
    parser = subparsers2.add_parser(
        'list',
        formatter_class=RawTextHelpFormatter,
        help="list shared space members of a shared space")
    parser.add_argument(
        'accounts', nargs="*",
        help="Filter documents by their account names")
    parser.add_argument('-u', '--uuid', dest="uuids", action="append", help="")
    parser.add_argument('-o', '--role', dest="roles", action="append",
                        choices=['ADMIN', 'WRITER', 'CONTRIBUTOR', 'READER'],
                        help="")
    add_list_parser_options(parser, delete=True, cdate=True)
    parser.set_defaults(__func__=ListCommand(config))

    # command : create
    parser = subparsers2.add_parser(
        'create', help="create shared space member for Workgroup")
    parser.add_argument('account', action="store", help="Account uuid")
    choices = ['ADMIN', 'WRITER', 'CONTRIBUTOR', 'READER']
    parser.add_argument(
            '--role', action="store",
            choices=choices,
            help="")
    parser.add_argument('--cli-mode', action="store_true", help="")
    parser.set_defaults(__func__=CreateCommand(config))

    # command : create-drive
    api_version = config.server.api_version.value
    if api_version >= 4.2:
        parser = subparsers2.add_parser(
            'create-drive', help="create shared space member for Drive.")
        parser.add_argument('account', action="store", help="Account uuid")
        choices=['DRIVE_ADMIN', 'DRIVE_WRITER', 'DRIVE_READER'],
        if api_version >= 5:
            choices=['WORKSPACE_ADMIN', 'WORKSPACE_WRITER', 'WORKSPACE_READER'],
        parser.add_argument(
                '--role', action="store",
                choices=choices,
                help="Drive role")
        parser.add_argument(
                '--nested-role', action="store",
                choices=['ADMIN', 'WRITER', 'CONTRIBUTOR', 'READER'],
                help="Nested role (default workgroup role)")
        parser.add_argument('--cli-mode', action="store_true", help="")
        parser.set_defaults(__func__=CreateDriveCommand(config))

    # command : delete
    parser = subparsers2.add_parser(
        'delete',
        help="delete shared space member")
    add_delete_parser_options(parser)
    parser.set_defaults(__func__=DeleteCommand(config))

    # command : update
    parser = subparsers2.add_parser(
        'update', help="update shared space member.")
    parser.add_argument(
        'uuid', action="store", help="").completer = Completer()
    parser.add_argument('--role', action="store",
                        choices=['ADMIN', 'WRITER', 'CONTRIBUTOR', 'READER'],
                        help="")
    parser.add_argument('--cli-mode', action="store_true", help="")
    parser.set_defaults(__func__=UpdateCommand(config))
